/* guignome/guignome_corba.c */

/* Copyright 2000 by Dr. David Alan Gilbert (elvis@treblig.org)      */
/*   based on guix11.c, with help from the gnome-hello code          */
/*   the Glade code, guys on the gnome-devel list, gmc,              */
/*   and the numbers 42 and 56. Chunks of this CORBA code are from   */
/*   gmc                                                             */
/* It may be distributed and used under the same license terms       */
/* as the rest of the Elvis package which it is part of.             */

#include <libgnorba/gnorba.h>
#include "elvisCorba.h"

/* The ORB for the whole program */
CORBA_ORB orb = CORBA_OBJECT_NIL;
      
/* The POA */
PortableServer_POA poa = CORBA_OBJECT_NIL;

/* Reference to the ElvisPrivate factory */
static CORBA_Object ElvisPrivate_server = CORBA_OBJECT_NIL;

/* Create a reference for our private server object */
static Elvis_ElvisPrivate 
ElvisPrivateServer_create (PortableServer_POA poa, CORBA_Environment *ev)
{
  TODO!

  POA_Elvis_ElvisPrivate__init ((PortableServer_Servant) eps, ev);
  objid = PortableServer_POA_activate_object (poa, eps, ev);
  CORBA_free (objid);

  return PortableServer_POA_servant_to_reference (poa, eps, ev);
}

/* Creates and registers the CORBA servers.  Returns TRUE on success, FALSE
 * otherwise.
 */
static int
register_servers (void)
{
  CORBA_Environment ev;
  int retval;
  int v;
  
  retval = FALSE;
  CORBA_exception_init (&ev);

  /* Register our private server and see if it was already there */
  ElvisPrivate_server = ElvisPrivateServer_create (poa, &ev);
  if (ev._major != CORBA_NO_EXCEPTION)
    goto out;

  v = goad_server_register (CORBA_OBJECT_NIL, ElvisPrivate_server,
          "IDL:Elvis:ElvisPrivate:1.0", "object", &ev);
  switch (v) {
  case 0:
    corba_have_server = FALSE;
    break;
  
  case -2:
    corba_have_server = FALSE;
    break;
  }

  /* Done */
 out:
  CORBA_exception_free (&ev);
  
  return retval;
}

/**
 * corba_init_server:
 * @void:
 * 
 * Initializes the CORBA server for GMC.  Returns whether initialization was
 * successful or not, and sets the global corba_have_server variable.
 *
 * Return value: TRUE if successful, FALSE otherwise.
 **/
int
corba_init_server (void)
{
  int retval;
  CORBA_Environment ev;

  retval = FALSE;
  CORBA_exception_init (&ev);
   
  /* Get the POA and create the server */

  poa = (PortableServer_POA) CORBA_ORB_resolve_initial_references (orb, "RootPOA", &ev);
  if (ev._major != CORBA_NO_EXCEPTION)
    goto out;
    
  CORBA_exception_free (&ev);

  /* See if the servers are there */
  ElvisPrivate_server = goad_server_activate_with_id (
    NULL,
    "IDL:Elvis:ElvisPrivate:1.0",
    GOAD_ACTIVATE_EXISTING_ONLY,
    NULL); 

  if (ElvisPrivate_server != CORBA_OBJECT_NIL) {
    corba_have_server = TRUE;
    retval = TRUE;
  } else
    retval = register_servers ();

 out:
  return retval;
}

/**
 * corba_activate_server:
 * @void:
 *
 * Activates the POA manager and thus makes the services available to the
 * outside world.
 **/
void
corba_activate_server (void)
{
  CORBA_Environment ev;
  PortableServer_POAManager poa_manager;
  
  /* Do nothing if the server is already running */
  if (corba_have_server)
    return;
   
  CORBA_exception_init (&ev);
    
  poa_manager = PortableServer_POA__get_the_POAManager (poa, &ev);
  if (ev._major != CORBA_NO_EXCEPTION)
    goto out;

  PortableServer_POAManager_activate (poa_manager, &ev);
  if (ev._major != CORBA_NO_EXCEPTION)
    goto out;

 out:

  CORBA_exception_free (&ev);
}
